#!/usr/bin/perl

use strict;
use warnings;

sub POE::Kernel::USE_SIGCHLD () { 1 }
sub POE::Kernel::ASSERT_USAGE () { 1 }
sub POE::Kernel::TRACE_SIGNALS () { 0 }

use Test::More;

use POE;
use POEx::ClockSkew;
use POSIX qw( SIGALRM );

sub DEBUG () { 0 }

my $interval = 10;

POEx::ClockSkew->spawn({ interval=>$interval, 
                            debug=>DEBUG, 
                           signal=>SIGALRM 
                      });

my $COUNT;
POE::Session->create( 
        inline_states => {
            _start => sub {
                $poe_kernel->sig( 'clock-skew' => 'skew' );
                $poe_kernel->alias_set( 'test' );
                diag( "Sleep ", $interval*2.5 );
                $poe_kernel->delay( after => $interval*2.5 );
            },
            _stop => sub {
                DEBUG and warn "stop";
            },
            after => sub {
                $COUNT = 0;
                # We can't adjust the system clock.  But we can fake a clock skew 
                # by sending a signal early.  This is going to provoke 2 clock-skew events.
                kill( SIGALRM, $$ );
                diag( "Sleep ", $interval*1.5 );
                $poe_kernel->delay( done => $interval*1.5 );
            },
            skew => sub {
                return unless defined $COUNT;
                my( $signal, $epsilon, $exepected ) = @_[ ARG0..$#_ ];
                $COUNT++;
                ok( abs($epsilon)<10, "Detected a fake skew" );
                
            },
            done => sub {
                $poe_kernel->signal( $poe_kernel, 'shutdown' );
            }
        }
    );

$poe_kernel->run;
pass( "Exited" );

ok( $COUNT, "Detected at least our fake skew" );

# We can't use an exact test count; we could have a real clock skew during testing.
done_testing();
